#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_form.interfaces.form import IWidgetForm, IFormHelp, IInnerSubForm
from pyams_portal.interfaces import IPortalContext, IPortalPage, IPortalTemplateConfiguration, \
    MANAGE_TEMPLATE_PERMISSION
from pyams_portal.zmi.interfaces import IPortalContextTemplatePropertiesMenu
from pyams_skin.interfaces import IPageHeader, IInnerPage
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_zmi.interfaces.menu import ISiteManagementMenu
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import IDataExtractedEvent

# import packages
from pyams_form.form import ajax_config
from pyams_form.group import NamedWidgetsGroup, FormWidgetsGroup
from pyams_form.help import FormHelp
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.zmi.layout import PortalTemplateLayoutView, PortalTemplatePortletEditForm, \
    PortalTemplatePortletAJAXEditForm
from pyams_portal.zmi.template import PortalTemplateHeaderAdapter
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm, InnerAdminEditForm
from pyams_zmi.zmi.site import PropertiesEditFormHeaderAdapter
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from z3c.form.browser.checkbox import SingleCheckBoxFieldWidget
from zope.interface import implementer, Interface, Invalid

from pyams_portal import _


#
# Portal context template configuration
#

@viewlet_config(name='template-properties.menu', context=IPortalContext, layer=IAdminLayer,
                manager=ISiteManagementMenu, permission=MANAGE_TEMPLATE_PERMISSION, weight=20)
@viewletmanager_config(name='template-properties.menu', layer=IAdminLayer, context=IPortalContext,
                       provides=IPortalContextTemplatePropertiesMenu)
@implementer(IPortalContextTemplatePropertiesMenu)
class PortalContextTemplatePropertiesMenu(MenuItem):
    """Portal context template properties menu"""

    label = _("Presentation")
    icon_class = 'fa-columns'
    url = '#template-properties.html'


class InheritedTemplateWidgetsGroup(FormWidgetsGroup):
    """Inherit template widgets groups"""


@adapter_config(context=(IPortalContext, IPyAMSLayer, InheritedTemplateWidgetsGroup), provides=IInnerSubForm)
class InheritedPortalContextTemplatePropertiesEditForm(InnerAdminEditForm):
    """Inherited portal context template properties"""

    def __new__(cls, context, request, group):
        if not group.form.getContent().can_inherit:
            return None
        return InnerAdminEditForm.__new__(cls)

    legend = None

    fields = field.Fields(IPortalPage).select('use_shared_template', 'shared_template')
    fields['use_shared_template'].widgetFactory = SingleCheckBoxFieldWidget

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    def getContent(self):
        return IPortalPage(self.context)

    def updateGroups(self):
        self.add_group(NamedWidgetsGroup(self, 'inherited_template_properties', self.widgets,
                                         ('use_shared_template', 'shared_template',),
                                         legend=_("Use shared template"),
                                         css_class='inner',
                                         switch=True,
                                         checkbox_switch=True,
                                         checkbox_field=IPortalPage['use_shared_template']))
        super(InheritedPortalContextTemplatePropertiesEditForm, self).updateGroups()


@subscriber(IDataExtractedEvent, form_selector=InheritedPortalContextTemplatePropertiesEditForm)
def handle_inherited_template_proterties_extract_event(event):
    """Handle template properties extract event"""
    form = event.form
    parent = form.parent_form  # parent group!
    form_data, form_errors = parent.form.extractData()
    if form_data.get('override_parent'):
        data = event.data
        if data.get('use_shared_template') and not data.get('shared_template'):
            parent.form.widgets.errors += (Invalid(_("You must choose to use a local template or select a shared "
                                                     "one!")),)


@pagelet_config(name='template-properties.html', context=IPortalContext, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
@ajax_config(name='template-properties.json', context=IPortalContext, layer=IPyAMSLayer)
@implementer(IWidgetForm, IInnerPage)
class PortalContextTemplatePropertiesEditForm(AdminEditForm):
    """Portal context template properties edit form"""

    @property
    def title(self):
        return self.context.title

    legend = _("Edit template configuration")
    override_legend = _("Override parent template")

    @property
    def fields(self):
        if self.getContent().can_inherit:
            fields = field.Fields(IPortalPage).select('override_parent')
            fields['override_parent'].widgetFactory = SingleCheckBoxFieldWidget
        else:
            fields = field.Fields(IPortalPage).select('use_shared_template', 'shared_template')
            fields['use_shared_template'].widgetFactory = SingleCheckBoxFieldWidget
        return fields

    edit_permission = MANAGE_TEMPLATE_PERMISSION

    def getContent(self):
        return IPortalPage(self.context)

    def updateGroups(self):
        if self.getContent().can_inherit:
            group = NamedWidgetsGroup(self, 'template_properties', self.widgets,
                                      ('override_parent',),
                                      legend=self.override_legend,
                                      css_class='inner',
                                      switch=True,
                                      checkbox_switch=True,
                                      checkbox_mode='disable',
                                      checkbox_field=IPortalPage['override_parent'],
                                      factory=InheritedTemplateWidgetsGroup)
        else:
            group = NamedWidgetsGroup(self, 'template_properties', self.widgets,
                                      ('use_shared_template', 'shared_template',),
                                      legend=_("Use shared template"),
                                      css_class='inner',
                                      switch=True,
                                      checkbox_switch=True,
                                      checkbox_field=IPortalPage['use_shared_template'])
        self.add_group(group)
        super(PortalContextTemplatePropertiesEditForm, self).updateGroups()

    def get_ajax_output(self, changes):
        output = super(self.__class__, self).get_ajax_output(changes)
        if changes:
            output.update({
                'status': 'redirect',
                'smallbox': {
                    'status': 'success',
                    'message': self.request.localizer.translate(self.successMessage)
                }
            })
        return output


@subscriber(IDataExtractedEvent, form_selector=PortalContextTemplatePropertiesEditForm)
def handle_template_proterties_extract_event(event):
    """Handle template properties extract event"""
    form = event.form
    if not form.getContent().can_inherit:
        data = event.data
        if data.get('use_shared_template') and not data.get('shared_template'):
            form.widgets.errors += (Invalid(_("You must choose to use a local template or select a shared one!")),)


@adapter_config(context=(Interface, IPyAMSLayer, PortalContextTemplatePropertiesEditForm), provides=IPageHeader)
class PortalContextPropertiesEditFormHeaderAdapter(PropertiesEditFormHeaderAdapter):
    """Portal context template properties edit form header adapter"""

    icon_class = 'fa fa-fw fa-columns'


@adapter_config(context=(IPortalContext, IAdminLayer, PortalContextTemplatePropertiesEditForm), provides=IFormHelp)
class PortalContextPropertiesEditFormHelpAdapter(FormHelp):
    """Portal context properties edit form help adapter"""

    message = _("If you choose a shared template, you can only adjust settings of "
                "each portlet individually but can't change portlets list or page configuration.\n"
                "If you use a local template, you can define a whole custom "
                "configuration but the template definition can't be reused anywhere...""")
    message_format = 'text'


#
#
#

@viewlet_config(name='template-config.menu', context=IPortalContext, layer=IAdminLayer,
                manager=IPortalContextTemplatePropertiesMenu, permission=MANAGE_TEMPLATE_PERMISSION, weight=50)
class PortalContextTemplateConfigMenu(MenuItem):
    """Portal context template configuration menu"""

    label = _("Template properties")

    url = '#template-config.html'

    def __new__(cls, context, request, view, manager=None):
        page = IPortalPage(context)
        if page.template is None:
            return None
        return MenuItem.__new__(cls)

    def get_url(self):
        page = IPortalPage(self.context)
        if page.use_local_template:
            return absolute_url(page.local_template, self.request, 'admin#properties.html')
        else:
            return super(PortalContextTemplateConfigMenu, self).get_url()


@pagelet_config(name='template-config.html', context=IPortalContext, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
class PortalContextTemplateLayoutView(PortalTemplateLayoutView):
    """Portal context template configuration view"""

    def get_template(self):
        return IPortalPage(self.context).template

    @property
    def can_change(self):
        if not IPortalPage(self.context).use_local_template:
            return False
        return self.request.has_permission(MANAGE_TEMPLATE_PERMISSION)


@adapter_config(context=(IPortalContext, IAdminLayer, PortalContextTemplateLayoutView), provides=IPageHeader)
class PortalContextTemplateLayoutHeaderAdapter(PortalTemplateHeaderAdapter):
    """Portal context template configuration header adapter"""


#
# Template management views
#

@view_config(name='get-slots-width.json', context=IPortalContext, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION, renderer='json', xhr=True)
def get_template_slots_width(request):
    """Get template slots width"""
    config = IPortalTemplateConfiguration(request.context)
    return config.get_slots_width(request.params.get('device'))


@view_config(name='portlet-properties.html', context=IPortalContext, request_type=IPyAMSLayer,
             permission=MANAGE_TEMPLATE_PERMISSION)
class PortalContextTemplatePortletEditForm(PortalTemplatePortletEditForm):
    """Portal context template portlet edit form"""


@view_config(name='portlet-properties.json', context=IPortalContext, request_type=IPyAMSLayer,
             permission=MANAGE_TEMPLATE_PERMISSION, renderer='json', xhr=True)
class PortalContextTemplatePortletAJAXEditForm(PortalTemplatePortletAJAXEditForm):
    """Portal context template portlet edit form, JSON renderer"""
